@@ -1,20 +1,8 @@
-/*************************************************************************************
- *  Copyright 2014-2016 Sebastian Kügler <sebas@kde.org>                             *
- *                                                                                   *
- *  This library is free software; you can redistribute it and/or                    *
- *  modify it under the terms of the GNU Lesser General Public                       *
- *  License as published by the Free Software Foundation; either                     *
- *  version 2.1 of the License, or (at your option) any later version.               *
- *                                                                                   *
- *  This library is distributed in the hope that it will be useful,                  *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                *
- *  Lesser General Public License for more details.                                  *
- *                                                                                   *
- *  You should have received a copy of the GNU Lesser General Public                 *
- *  License along with this library; if not, write to the Free Software              *
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA       *
- *************************************************************************************/
+/*
+ *  SPDX-FileCopyrightText: 2014-2016 Sebastian Kügler <sebas@kde.org>
+ *
+ *  SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #include "doctor.h"
 #include "dpmsclient.h"
@@ -81,7 +69,7 @@ void Doctor::start(QCommandLineParser *parser)
     if (m_parser->isSet(QStringLiteral("info"))) {
         showBackends();
     }
-    if (parser->isSet(QStringLiteral("json")) || parser->isSet(QStringLiteral("outputs")) || !m_positionalArgs.isEmpty()) {
+    if (parser->isSet(QStringLiteral("json")) || parser->isSet(QStringLiteral("outputs")) || !m_outputArgs.isEmpty()) {
         KScreen::GetConfigOperation *op = new KScreen::GetConfigOperation();
         connect(op, &KScreen::GetConfigOperation::finished, this, [this](KScreen::ConfigOperation *op) {
             configReceived(op);
@@ -90,7 +78,7 @@ void Doctor::start(QCommandLineParser *parser)
     }
     if (m_parser->isSet(QStringLiteral("dpms"))) {
         if (!QGuiApplication::platformName().startsWith(QLatin1String("wayland"))) {
-            cerr << "DPMS is only supported on Wayland." << endl;
+            cerr << "DPMS is only supported on Wayland." << Qt::endl;
             // We need to kick the event loop, otherwise .quit() hangs
             QTimer::singleShot(0, qApp->quit);
             return;
@@ -155,45 +143,45 @@ void Doctor::showDpms()
 
 void Doctor::showBackends() const
 {
-    cout << "Environment: " << endl;
+    cout << "Environment: " << Qt::endl;
     auto env_kscreen_backend = (qgetenv("KSCREEN_BACKEND").isEmpty()) ? QStringLiteral("[not set]") : QString::fromUtf8(qgetenv("KSCREEN_BACKEND"));
-    cout << "  * KSCREEN_BACKEND           : " << env_kscreen_backend << endl;
+    cout << "  * KSCREEN_BACKEND           : " << env_kscreen_backend << Qt::endl;
     auto env_kscreen_backend_inprocess =
         (qgetenv("KSCREEN_BACKEND_INPROCESS").isEmpty()) ? QStringLiteral("[not set]") : QString::fromUtf8(qgetenv("KSCREEN_BACKEND_INPROCESS"));
-    cout << "  * KSCREEN_BACKEND_INPROCESS : " << env_kscreen_backend_inprocess << endl;
+    cout << "  * KSCREEN_BACKEND_INPROCESS : " << env_kscreen_backend_inprocess << Qt::endl;
     auto env_kscreen_logging = (qgetenv("KSCREEN_LOGGING").isEmpty()) ? QStringLiteral("[not set]") : QString::fromUtf8(qgetenv("KSCREEN_LOGGING"));
-    cout << "  * KSCREEN_LOGGING           : " << env_kscreen_logging << endl;
+    cout << "  * KSCREEN_LOGGING           : " << env_kscreen_logging << Qt::endl;
 
-    cout << "Logging to                : " << (Log::instance()->enabled() ? Log::instance()->logFile() : QStringLiteral("[logging disabled]")) << endl;
-    auto backends = BackendManager::instance()->listBackends();
+    cout << "Logging to                : " << (Log::instance()->enabled() ? Log::instance()->logFile() : QStringLiteral("[logging disabled]")) << Qt::endl;
+    const auto backends = BackendManager::instance()->listBackends();
     auto preferred = BackendManager::instance()->preferredBackend();
-    cout << "Preferred KScreen backend : " << green << preferred.fileName() << cr << endl;
-    cout << "Available KScreen backends:" << endl;
-    Q_FOREACH (const QFileInfo f, backends) {
+    cout << "Preferred KScreen backend : " << green << preferred.fileName() << cr << Qt::endl;
+    cout << "Available KScreen backends:" << Qt::endl;
+    for (const QFileInfo &f : backends) {
         auto c = blue;
         if (preferred == f) {
             c = green;
         }
-        cout << "  * " << c << f.fileName() << cr << ": " << f.absoluteFilePath() << endl;
+        cout << "  * " << c << f.fileName() << cr << ": " << f.absoluteFilePath() << Qt::endl;
     }
-    cout << endl;
+    cout << Qt::endl;
 }
 
-void Doctor::setOptionList(const QStringList &positionalArgs)
+void Doctor::setOptionList(const QStringList &outputArgs)
 {
-    m_positionalArgs = positionalArgs;
+    m_outputArgs = outputArgs;
 }
 
-void Doctor::parsePositionalArgs()
+void Doctor::parseOutputArgs()
 {
     // qCDebug(KSCREEN_DOCTOR) << "POSARGS" << m_positionalArgs;
-    Q_FOREACH (const QString &op, m_positionalArgs) {
+    for (const QString &op : qAsConst(m_outputArgs)) {
         auto ops = op.split(QLatin1Char('.'));
         if (ops.count() > 2) {
             bool ok;
             int output_id = -1;
             if (ops[0] == QLatin1String("output")) {
-                Q_FOREACH (const auto &output, m_config->outputs()) {
+                for (const auto &output : m_config->outputs()) {
                     if (output->name() == ops[1]) {
                         output_id = output->id();
                     }
@@ -201,7 +189,7 @@ void Doctor::parsePositionalArgs()
                 if (output_id == -1) {
                     output_id = ops[1].toInt(&ok);
                     if (!ok) {
-                        cerr << "Unable to parse output id: " << ops[1] << endl;
+                        cerr << "Unable to parse output id: " << ops[1] << Qt::endl;
                         qApp->exit(3);
                         return;
                     }
@@ -235,7 +223,7 @@ void Doctor::parsePositionalArgs()
                     int x = _pos[0].toInt(&ok);
                     int y = _pos[1].toInt(&ok);
                     if (!ok) {
-                        cerr << "Unable to parse position: " << ops[3] << endl;
+                        cerr << "Unable to parse position: " << ops[3] << Qt::endl;
                         qApp->exit(5);
                         return;
                     }
@@ -278,8 +266,39 @@ void Doctor::parsePositionalArgs()
                         qApp->exit(9);
                         return;
                     }
+                } else if (ops.count() == 4 && ops[2] == QLatin1String("overscan")) {
+                    const uint32_t overscan = ops[3].toInt();
+                    if (overscan > 100) {
+                        qCWarning(KSCREEN_DOCTOR) << "Wrong input: allowed values for overscan are from 0 to 100";
+                        qApp->exit(9);
+                        return;
+                    }
+                    if (!setOverscan(output_id, overscan)) {
+                        qCDebug(KSCREEN_DOCTOR) << "Could not set overscan " << overscan << " to output " << output_id;
+                        qApp->exit(9);
+                        return;
+                    }
+                } else if (ops.count() == 4 && ops[2] == QLatin1String("vrrpolicy")) {
+                    const QString _policy = ops[3].toLower();
+                    KScreen::Output::VrrPolicy policy;
+                    if (_policy == QStringLiteral("never")) {
+                        policy = KScreen::Output::VrrPolicy::Never;
+                    } else if (_policy == QStringLiteral("always")) {
+                        policy = KScreen::Output::VrrPolicy::Always;
+                    } else if (_policy == QStringLiteral("automatic")) {
+                        policy = KScreen::Output::VrrPolicy::Automatic;
+                    } else {
+                        qCDebug(KSCREEN_DOCTOR) << "Wrong input: Only allowed values are \"never\", \"always\" and \"automatic\"";
+                        qApp->exit(9);
+                        return;
+                    }
+                    if (!setVrrPolicy(output_id, policy)) {
+                        qCDebug(KSCREEN_DOCTOR) << "Could not set vrr policy " << policy << " to output " << output_id;
+                        qApp->exit(9);
+                        return;
+                    }
                 } else {
-                    cerr << "Unable to parse arguments: " << op << endl;
+                    cerr << "Unable to parse arguments: " << op << Qt::endl;
                     qApp->exit(2);
                     return;
                 }
@@ -301,7 +320,7 @@ void Doctor::configReceived(KScreen::ConfigOperation *op)
         qApp->quit();
     }
 
-    parsePositionalArgs();
+    parseOutputArgs();
 
     if (m_changed) {
         applyConfig();
@@ -342,7 +361,7 @@ void Doctor::showOutputs() const
     typeString[KScreen::Output::TVC4] = QStringLiteral("TVC4");
     typeString[KScreen::Output::DisplayPort] = QStringLiteral("DisplayPort");
 
-    Q_FOREACH (const auto &output, m_config->outputs()) {
+    for (const auto &output : m_config->outputs()) {
         cout << green << "Output: " << cr << output->id() << " " << output->name();
         cout << " " << (output->isEnabled() ? green + QLatin1String("enabled") : red + QLatin1String("disabled"));
         cout << " " << (output->isConnected() ? green + QLatin1String("connected") : red + QLatin1String("disconnected"));
@@ -350,7 +369,7 @@ void Doctor::showOutputs() const
         auto _type = typeString[output->type()];
         cout << " " << yellow << (_type.isEmpty() ? QStringLiteral("UnmappedOutputType") : _type);
         cout << blue << " Modes: " << cr;
-        Q_FOREACH (auto mode, output->modes()) {
+        for (const auto &mode : output->modes()) {
             auto name = QStringLiteral("%1x%2@%3")
                             .arg(QString::number(mode->size().width()), QString::number(mode->size().height()), QString::number(qRound(mode->refreshRate())));
             if (mode == output->currentMode()) {
@@ -365,10 +384,26 @@ void Doctor::showOutputs() const
         cout << yellow << "Geometry: " << cr << g.x() << "," << g.y() << " " << g.width() << "x" << g.height() << " ";
         cout << yellow << "Scale: " << cr << output->scale() << " ";
         cout << yellow << "Rotation: " << cr << output->rotation() << " ";
+        cout << yellow << "Overscan: " << cr << output->overscan() << " ";
+        cout << yellow << "Vrr: ";
+        if (output->capabilities() & Output::Capability::Vrr) {
+            switch (output->vrrPolicy()) {
+            case Output::VrrPolicy::Never:
+                cout << cr << "Never ";
+                break;
+            case Output::VrrPolicy::Automatic:
+                cout << cr << "Automatic ";
+                break;
+            case Output::VrrPolicy::Always:
+                cout << cr << "Always ";
+            }
+        } else {
+            cout << cr << "incapable ";
+        }
         if (output->isPrimary()) {
             cout << blue << "primary";
         }
-        cout << cr << endl;
+        cout << cr << Qt::endl;
     }
 }
 
@@ -385,15 +420,15 @@ bool Doctor::setEnabled(int id, bool enabled = true)
         return false;
     }
 
-    Q_FOREACH (const auto &output, m_config->outputs()) {
+    for (const auto &output : m_config->outputs()) {
         if (output->id() == id) {
-            cout << (enabled ? "Enabling " : "Disabling ") << "output " << id << endl;
+            cout << (enabled ? "Enabling " : "Disabling ") << "output " << id << Qt::endl;
             output->setEnabled(enabled);
             m_changed = true;
             return true;
         }
     }
-    cerr << "Output with id " << id << " not found." << endl;
+    cerr << "Output with id " << id << " not found." << Qt::endl;
     qApp->exit(8);
     return false;
 }
@@ -405,7 +440,7 @@ bool Doctor::setPosition(int id, const QPoint &pos)
         return false;
     }
 
-    Q_FOREACH (const auto &output, m_config->outputs()) {
+    for (const auto &output : m_config->outputs()) {
         if (output->id() == id) {
             qCDebug(KSCREEN_DOCTOR) << "Set output position" << pos;
             output->setPos(pos);
@@ -413,7 +448,7 @@ bool Doctor::setPosition(int id, const QPoint &pos)
             return true;
         }
     }
-    cout << "Output with id " << id << " not found." << endl;
+    cout << "Output with id " << id << " not found." << Qt::endl;
     return false;
 }
 
@@ -424,10 +459,10 @@ bool Doctor::setMode(int id, const QString &mode_id)
         return false;
     }
 
-    Q_FOREACH (const auto &output, m_config->outputs()) {
+    for (const auto &output : m_config->outputs()) {
         if (output->id() == id) {
             // find mode
-            Q_FOREACH (const KScreen::ModePtr mode, output->modes()) {
+            for (const KScreen::ModePtr mode : output->modes()) {
                 auto name =
                     QStringLiteral("%1x%2@%3")
                         .arg(QString::number(mode->size().width()), QString::number(mode->size().height()), QString::number(qRound(mode->refreshRate())));
@@ -440,7 +475,7 @@ bool Doctor::setMode(int id, const QString &mode_id)
             }
         }
     }
-    cout << "Output mode " << mode_id << " not found." << endl;
+    cout << "Output mode " << mode_id << " not found." << Qt::endl;
     return false;
 }
 
@@ -451,14 +486,14 @@ bool Doctor::setScale(int id, qreal scale)
         return false;
     }
 
-    Q_FOREACH (const auto &output, m_config->outputs()) {
+    for (const auto &output : m_config->outputs()) {
         if (output->id() == id) {
             output->setScale(scale);
             m_changed = true;
             return true;
         }
     }
-    cout << "Output scale " << id << " invalid." << endl;
+    cout << "Output scale " << id << " invalid." << Qt::endl;
     return false;
 }
 
@@ -469,14 +504,50 @@ bool Doctor::setRotation(int id, KScreen::Output::Rotation rot)
         return false;
     }
 
-    Q_FOREACH (const auto &output, m_config->outputs()) {
+    for (const auto &output : m_config->outputs()) {
         if (output->id() == id) {
             output->setRotation(rot);
             m_changed = true;
             return true;
         }
     }
-    cout << "Output rotation " << id << " invalid." << endl;
+    cout << "Output rotation " << id << " invalid." << Qt::endl;
+    return false;
+}
+
+bool Doctor::setOverscan(int id, uint32_t overscan)
+{
+    if (!m_config) {
+        qCWarning(KSCREEN_DOCTOR) << "Invalid config.";
+        return false;
+    }
+
+    for (const auto &output : m_config->outputs()) {
+        if (output->id() == id) {
+            output->setOverscan(overscan);
+            m_changed = true;
+            return true;
+        }
+    }
+    cout << "Output overscan " << id << " invalid." << Qt::endl;
+    return false;
+}
+
+bool Doctor::setVrrPolicy(int id, KScreen::Output::VrrPolicy policy)
+{
+    if (!m_config) {
+        qCWarning(KSCREEN_DOCTOR) << "Invalid config.";
+        return false;
+    }
+
+    for (const auto &output : m_config->outputs()) {
+        if (output->id() == id) {
+            output->setVrrPolicy(policy);
+            m_changed = true;
+            return true;
+        }
+    }
+    cout << "Output VrrPolicy " << id << " invalid." << Qt::endl;
     return false;
 }
 
